@@ -45,6 +45,7 @@
 #include <ImfKeyCodeAttribute.h>
 #include <ImfLineOrderAttribute.h>
 #include <ImfMatrixAttribute.h>
+#include <ImfOpaqueAttribute.h>
 #include <ImfPreviewImageAttribute.h>
 #include <ImfRationalAttribute.h>
 #include <ImfStringAttribute.h>
@@ -339,7 +340,10 @@ PyPart::readPixels(MultiPartInputFile& infile, const ChannelList& channel_list,
     InputPart part (infile, part_index);
 
     part.setFrameBuffer (frameBuffer);
-    part.readPixels (dw.min.y, dw.max.y);
+    {
+        py::gil_scoped_release release;
+        part.readPixels (dw.min.y, dw.max.y);
+    }
 }
 
 void
@@ -582,11 +586,17 @@ PyPart::readDeepPixels(MultiPartInputFile& infile, const std::string& type, cons
     {
         DeepScanLineInputPart part (infile, part_index);
         part.setFrameBuffer (frameBuffer);
-        part.readPixelSampleCounts (dw.min.y, dw.max.y);
+        {
+            py::gil_scoped_release release;
+            part.readPixelSampleCounts (dw.min.y, dw.max.y);
+        }
 
         setDeepSliceData(channel_list, height, width, sliceDataMap, rgbaChannelMap, sampleCount);
 
-        part.readPixels (dw.min.y, dw.max.y);
+        {
+            py::gil_scoped_release release;
+            part.readPixels (dw.min.y, dw.max.y);
+        }
     }
     else if (type == DEEPTILE)
     {
@@ -596,11 +606,17 @@ PyPart::readDeepPixels(MultiPartInputFile& infile, const std::string& type, cons
         int numXTiles = part.numXTiles (0);
         int numYTiles = part.numYTiles (0);
 
-        part.readPixelSampleCounts (0, numXTiles - 1, 0, numYTiles - 1);
+        {
+            py::gil_scoped_release release;
+            part.readPixelSampleCounts (0, numXTiles - 1, 0, numYTiles - 1);
+        }
 
         setDeepSliceData(channel_list, height, width, sliceDataMap, rgbaChannelMap, sampleCount);
 
-        part.readTiles (0, numXTiles - 1, 0, numYTiles - 1);
+        {
+            py::gil_scoped_release release;
+            part.readTiles (0, numXTiles - 1, 0, numYTiles - 1);
+        }
     }
 }
 
@@ -685,13 +701,19 @@ PyPart::writePixels(MultiPartOutputFile& outfile, const Box2i& dw) const
     {
         OutputPart part(outfile, part_index);
         part.setFrameBuffer (frameBuffer);
-        part.writePixels (height());
+        {
+            py::gil_scoped_release release;
+            part.writePixels (height());
+        }
     }
     else
     {
         TiledOutputPart part(outfile, part_index);
         part.setFrameBuffer (frameBuffer);
-        part.writeTiles (0, part.numXTiles() - 1, 0, part.numYTiles() - 1);
+        {
+            py::gil_scoped_release release;
+            part.writeTiles (0, part.numXTiles() - 1, 0, part.numYTiles() - 1);
+        }
     }
 }
 
@@ -880,16 +902,22 @@ PyPart::writeDeepPixels(MultiPartOutputFile& outfile, const Box2i& dw) const
     {
         DeepScanLineOutputPart part(outfile, part_index);
         part.setFrameBuffer (frameBuffer);
-        part.writePixels (height);
+        {
+            py::gil_scoped_release release;
+            part.writePixels (height);
+        }
     }
     else 
     {
         DeepTiledOutputPart part(outfile, part_index);
         part.setFrameBuffer (frameBuffer);
 
-        for (int y = 0; y < part.numYTiles (0); y++)
-            for (int x = 0; x < part.numXTiles (0); x++)
-                part.writeTile (x, y, 0);
+        {
+            py::gil_scoped_release release;
+            for (int y = 0; y < part.numYTiles (0); y++)
+                for (int x = 0; x < part.numXTiles (0); x++)
+                    part.writeTile (x, y, 0);
+        }
     }
 }
 
@@ -1180,25 +1208,37 @@ PyFile::write(const char* outfilename)
             {
                 InputPart  inPart (*_inputFile, p);
                 OutputPart outPart (outfile, p);
-                outPart.copyPixels (inPart);
+                {
+                    py::gil_scoped_release release;
+                    outPart.copyPixels (inPart);
+                }
             }
             else if (type == TILEDIMAGE)
             {
                 TiledInputPart  inPart (*_inputFile, p);
                 TiledOutputPart outPart (outfile, p);
-                outPart.copyPixels (inPart);
+                {
+                    py::gil_scoped_release release;
+                    outPart.copyPixels (inPart);
+                }
             }
             else if (type == DEEPSCANLINE)
             {
                 DeepScanLineInputPart  inPart (*_inputFile, p);
                 DeepScanLineOutputPart outPart (outfile, p);
-                outPart.copyPixels (inPart);
+                {
+                    py::gil_scoped_release release;
+                    outPart.copyPixels (inPart);
+                }
             }
             else if (type == DEEPTILE)
             {
                 DeepTiledInputPart  inPart (*_inputFile, p);
                 DeepTiledOutputPart outPart (outfile, p);
-                outPart.copyPixels (inPart);
+                {
+                    py::gil_scoped_release release;
+                    outPart.copyPixels (inPart);
+                }
             }
         }
     }
@@ -1336,7 +1376,13 @@ PyFile::getAttributeObject(const std::string& name, const Attribute* a)
         return py::cast(*v);
     }
 
-    if (auto v = dynamic_cast<const ChannelListAttribute*> (a))
+ 
+    if (auto v = dynamic_cast<const OpaqueAttribute*> (a))
+    {
+        return py::cast(*v);
+    }
+
+   if (auto v = dynamic_cast<const ChannelListAttribute*> (a))
     {
         auto L = v->value();
         auto l = py::list();
@@ -1549,7 +1595,7 @@ PyFile::getAttributeObject(const std::string& name, const Attribute* a)
     
     if (auto v = dynamic_cast<const V3dAttribute*> (a))
         return make_v3(v->value());
-    
+
     std::stringstream err;
     err << "unsupported attribute type: " << a->typeName();
     throw std::runtime_error(err.str());
@@ -1979,6 +2025,10 @@ PyFile::insertAttribute(Header& header, const std::string& name, const py::objec
     {
         header.insert(name, py::cast<Imf::BytesAttribute>(object));
     }
+    else if (py::isinstance<Imf::OpaqueAttribute>(object))
+    {
+        header.insert(name, py::cast<Imf::OpaqueAttribute>(object));
+    }
     else if (auto v = py_cast<Compression>(object))
         header.insert(name, CompressionAttribute(static_cast<Compression>(*v)));
     else if (auto v = py_cast<Envmap>(object))
@@ -2298,9 +2348,16 @@ repr(const T& v)
     s << v;
     return s.str();
 }
-
 } // namespace
 
+OPENEXR_IMF_INTERNAL_NAMESPACE_HEADER_ENTER
+inline bool
+operator==(const Imf::OpaqueAttribute& a,const Imf::OpaqueAttribute& b)
+{
+ return(string(a.typeName()) == string(b.typeName()) && a.dataSize()==b.dataSize()
+		&& (a.dataSize()==0 || memcmp(&a.data()[0],&b.data()[0],a.dataSize())==0));
+}
+OPENEXR_IMF_INTERNAL_NAMESPACE_HEADER_EXIT
 
 PYBIND11_MODULE(OpenEXR, m)
 {
@@ -2416,6 +2473,36 @@ PYBIND11_MODULE(OpenEXR, m)
                 + std::to_string(self.size()) + " bytes), "
                 + "type_hint='" + self.typeHint + "'>");
         });
+ 
+    py::class_<OpaqueAttribute>(m, "OpaqueAttribute")
+        .def(py::init([](py::bytes data, std::string typeName) {
+            std::string_view data_view(data);
+
+            return std::make_unique<Imf::OpaqueAttribute>(
+	        typeName.c_str(),
+                data_view.size(),
+                reinterpret_cast<const unsigned char*>(data_view.data())
+            );
+        }), py::arg("data"), py::arg("typeName") = "")
+        .def_property_readonly("data",
+            [](const OpaqueAttribute& self) {
+            const auto& data = self.data();
+            const auto& size = self.dataSize();
+            const char* ptr = (size == 0) ?
+                nullptr : reinterpret_cast<const char*>(&data[0]);
+            return py::bytes(ptr, size);
+            })
+        .def_property_readonly("typeName",
+	    [](const OpaqueAttribute& self) {
+   	    return self.typeName();
+	     })
+        .def(py::self == py::self)
+        .def("__repr__", [](const OpaqueAttribute& self) {
+            return (
+                "<OpaqueAttribute type="+string(self.typeName())+", data=b'...' ("
+                + std::to_string(self.dataSize()) + " bytes)>");
+        });
+
 
     py::class_<TileDescription>(m, "TileDescription", "Tile description for tiled images")
         .def(py::init())
